import base64
import hashlib
import time

from Crypto import Random
from Crypto.Cipher import AES
from pysine import sine


class Note(object):
    C = 'C'
    Db = 'Db'
    D = 'D'
    Eb = 'Eb'
    E = 'E'
    F = 'F'
    Gb = 'Gb'
    G = 'G'
    Ab = 'Ab'
    A = 'A'
    Bb = 'Bb'
    B = 'B'


class Notes(object):

    __notes = [Note.C, Note.Db, Note.D, Note.Eb, Note.E, Note.F, Note.Gb, Note.G,
               Note.Ab, Note.A, Note.Bb, Note.B]

    @staticmethod
    def get(index):
        return Notes.__notes[index]

    @staticmethod
    def get_freq(note):
        freqs = {
            Note.C: 523,
            Note.Db: 554,
            Note.D: 587,
            Note.Eb: 622,
            Note.E: 659,
            Note.F: 698,
            Note.Gb: 739,
            Note.G: 783,
            Note.Ab: 830,
            Note.A: 880,
            Note.Bb: 932,
            Note.B: 987
        }
        return freqs[note]

    def __str__(self):
        notes_str = ''
        i = 0
        for note in Notes.__notes:
            notes_str += '%s - %s\n' % (i, note)
            i += 1
        return notes_str


def begin(sequence):
    for note in sequence:
        sine(Notes.get_freq(note), 0.5)
        time.sleep(0.1)
    print('%s\n%s\nChoose next note:'
          % (sequence, Notes()))


def play(sequence):
    print(sequence)

    sine(Notes.get_freq(sequence[-1]), 0.5)

    resp = '7d708279cac91289d4120880b99b0264d5faeb4ca307298ab5497e69d2229c46'
    msg_enc = '0zjKLmOPTohY4f4zbzie/+IKYPGN7NROzzctskPAL3Q='

    sequence_str = ''.join(sequence)

    m = hashlib.sha3_256()
    m.update(sequence_str.encode())
    if m.digest().hex() == resp:
        key = sequence_str + resp
        time.sleep(1)
        for note in sequence:
            sine(Notes.get_freq(note), 0.5)
            time.sleep(0.1)

        return decrypt(key, msg_enc)
    else:
        if len(sequence) < 15:
            return ''
        else:
            return 'Try again!'


def choose():
    i = int(input('Note: '))
    return Notes.get(i)


def generate_encrypted_msg(key, msg):
    key = hashlib.sha256(key.encode()).digest()
    bs = AES.block_size
    raw = _pad(msg, bs)
    iv = Random.new().read(bs)
    cipher = AES.new(key, AES.MODE_CBC, iv)
    return base64.b64encode(iv + cipher.encrypt(raw.encode()))


def decrypt(key, msg):
    key = hashlib.sha256(key.encode()).digest()
    msg = base64.b64decode(msg)
    iv = msg[:AES.block_size]
    cipher = AES.new(key, AES.MODE_CBC, iv)
    return _unpad(cipher.decrypt(msg[AES.block_size:])).decode('utf-8')


def _pad(s, bs):
    return s + (bs - len(s) % bs) * chr(bs - len(s) % bs)


def _unpad(s):
    return s[:-ord(s[len(s) - 1:])]
